"""
    Module containing functions for controlling the device connection and status
"""

# import necessary modules
import ctypes as ctp
import sys


class state:
    # device parameters
    dwf = None
    hdwf = None
    error = False
    error_msg = None


def load():
    # load the WaveForms SDK
    if sys.platform.startswith("win"):
        state.dwf = ctp.cdll.dwf
    elif sys.platform.startswith("darwin"):
        state.dwf = ctp.cdll.LoadLibrary(
            "/Library/Frameworks/dwf.framework/dwf")
    else:
        state.dwf = ctp.cdll.LoadLibrary("libdwf.so")

    # check for errors
    check_error()
    if state.error:
        print(state.error_msg)
        quit()
    return state.dwf


def check_error():
    # set error flag and error message if necessary
    szerr = ctp.create_string_buffer(512)
    state.dwf.FDwfGetLastErrorMsg(szerr)
    if szerr[0] != b'\0':
        state.error = True
        state.error_msg = str(szerr.value)
    return


def open():
    # count devices
    cDevice = ctp.c_int()
    state.dwf.FDwfEnum(ctp.c_int(0), ctp.byref(cDevice))
    if cDevice.value <= 0:
        state.error = True
        state.error_msg = "No device is connected"
        print(state.error_msg)
        quit()

    # search for Analog Discovery Studio
    devicename = ctp.create_string_buffer(64)
    index = -1
    for iDevice in range(0, cDevice.value):
        state.dwf.FDwfEnumDeviceName(ctp.c_int(iDevice), devicename)
        if str(devicename.value) == "b'Analog Discovery Studio'":
            index = iDevice
            break
    if index < 0:
        state.error = True
        state.error_msg = "Analog Discovery Studio isn't connected"
        print(state.error_msg)
        quit()

    # open device
    hdwf = ctp.c_int()
    state.dwf.FDwfDeviceOpen(ctp.c_int(index), ctp.byref(hdwf))
    state.hdwf = hdwf
    check_error()
    if state.error:
        print(state.error_msg)
        quit()

    # set dynamic auto configuration
    state.dwf.FDwfDeviceAutoConfigureSet(state.hdwf, ctp.c_int(3))
    return state.hdwf


def close():
    # reset every instrument
    state.dwf.FDwfAnalogInReset(state.hdwf)
    state.dwf.FDwfAnalogOutReset(state.hdwf, ctp.c_int(-1))
    state.dwf.FDwfAnalogIOReset(state.hdwf)
    state.dwf.FDwfDigitalInReset(state.hdwf)
    state.dwf.FDwfDigitalOutReset(state.hdwf)
    state.dwf.FDwfDigitalIOReset(state.hdwf)

    # close the device
    state.dwf.FDwfDeviceClose(state.hdwf)
    return
